#include <assert.h>
#include <tk.h>

#include <hygraph/Hygraph.h>
#include <util/MN_Point.h>
#include <util/MN_Color.h>
#include <util/unix.h>

#include "widget.h"

//------------------------------------------------------------------------------

static Tcl_Interp* interp;
static char* canvas;
static char buf[256];

//------------------------------------------------------------------------------

void InitMarkLib( Tcl_Interp* i, char* c )
{
   interp = i;
   canvas = strdup( c );
}

//==============================================================================

Meter::Meter( Vertex* v, MN_Point* p, MN_Point* s, char* l )
: Mark( v, p, s, l )
{
   sprintf( buf,
      "%s create rectangle %g %g %g %g -fill #%02x%02x%02x -tag node",
      canvas, pos.x, pos.y, pos.x + size.x, pos.y + size.y,
      (int)(255 * blank.x), (int)(255 * blank.y), (int)(255 * blank.z) );
   Tcl_GlobalEval( interp, buf );
   big_rect = atoi( interp->result );

   sprintf( buf,
      "%s create rectangle %g %g %g %g -fill #%02x%02x%02x -tag node",
      canvas, pos.x, pos.y, pos.x + size.x, pos.y + size.y,
      (int)(255 * fill.x), (int)(255 * fill.y), (int)(255 * fill.z) );
   Tcl_GlobalEval( interp, buf );
   small_rect = atoi( interp->result );

   sprintf( buf, "%s create text %g %g -text {%s} -anchor nw -font $config(font,$state(size))",
      canvas, pos.x + size.x + 5, pos.y, label );
   Tcl_GlobalEval( interp, buf );
   label_widget = atoi( interp->result );

   sprintf( buf, "set idFromW(%d) %x", big_rect, (unsigned int)v );
   Tcl_GlobalEval( interp, buf );
};

inline void ShowWidget( char* c, Tcl_Interp* i, int w, bool show )
{
   if( show )
      sprintf( buf, "%s move %d  10000  10000", c, w );
   else
      sprintf( buf, "%s move %d -10000 -10000", c, w );
   Tcl_GlobalEval( i, buf );
};

virtual void Meter::Show( bool show )
{
   if( show )
   {
      if( pos.x < 0 )
      {
         ShowWidget( canvas, interp, big_rect,     true );
         ShowWidget( canvas, interp, small_rect,   true );
         ShowWidget( canvas, interp, label_widget, true );
      }
   }
   else   // hide
   {
      if( pos.x >= 0 )
      {
         ShowWidget( canvas, interp, big_rect,     false );
         ShowWidget( canvas, interp, small_rect,   false );
         ShowWidget( canvas, interp, label_widget, false );
      }
   }
};

virtual void Meter::SetPos( MN_Point *p )
{
   pos = *p;
   Display();
};

virtual void Meter::SetSize( MN_Point *p )
{
   size = *p;
   Display();
};

virtual void Meter::SetFill( MN_Color *p )
{
   fill = *p;
   sprintf( buf, "%s itemconfigure %d -fill #%02x%02x%02x",
      canvas, small_rect, fill.x, fill.y, fill.z );
   Tcl_GlobalEval( interp, buf );
};

virtual void Meter::SetBlank( MN_Color *p )
{
   blank = *p;
   sprintf( buf, "%s itemconfigure %d -fill #%02x%02x%02x",
      canvas, big_rect, blank.x, blank.y, blank.z );
   Tcl_GlobalEval( interp, buf );
};

virtual void Meter::SetOutline( MN_Color *p )
{
   outline = *p;
   sprintf( buf, "%s itemconfigure %d -outline #%02x%02x%02x",
      canvas, big_rect, outline.x, outline.y, outline.z );
   Tcl_GlobalEval( interp, buf );
};

virtual void Meter::SetThickness( float p )
{
   thickness = p;
   sprintf( buf, "%s itemconfigure %d -width %g",
      canvas, big_rect, 1.0 + 5.0 * p );
   Tcl_GlobalEval( interp, buf );
};

//==============================================================================

virtual void VMeter::Display( void )
{
   sprintf( buf, "%s coords %d %g %g %g %g",
      canvas, big_rect, pos.x, pos.y, pos.x + size.x, pos.y + size.y );
   Tcl_GlobalEval( interp, buf );

   sprintf( buf, "%s coords %d %g %g %g %g",
      canvas, small_rect, pos.x, pos.y + (1.0 - fillPercentage) * size.y,
      pos.x + size.x, pos.y + size.y );
   Tcl_GlobalEval( interp, buf );

   sprintf( buf, "%s coords %d %g %g %g %g",
      canvas, label_widget, pos.x + size.x + 5, pos.y );
   Tcl_GlobalEval( interp, buf );
};

virtual void VMeter::SetFillPercentage( float p )
{
   fillPercentage = p;
   sprintf( buf, "%s coords %d %g %g %g %g",
      canvas, small_rect, pos.x, pos.y + (1.0 - fillPercentage) * size.y,
      pos.x + size.x, pos.y + size.y );
   Tcl_GlobalEval( interp, buf );
};

//==============================================================================

virtual void HMeter::Display( void )
{
   sprintf( buf, "%s coords %d %g %g %g %g",
      canvas, big_rect, pos.x, pos.y, pos.x + size.x, pos.y + size.y );
   Tcl_GlobalEval( interp, buf );

   sprintf( buf, "%s coords %d %g %g %g %g",
      canvas, small_rect, pos.x, pos.y, pos.x + fillPercentage * size.x,
      pos.y + size.y );
   Tcl_GlobalEval( interp, buf );

   sprintf( buf, "%s coords %d %g %g",
      canvas, label_widget, pos.x + size.x, pos.y );
   Tcl_GlobalEval( interp, buf );
};

virtual void HMeter::SetFillPercentage( float p )
{
   fillPercentage = p;
   sprintf( buf, "%s coords %d %g %g %g %g",
      canvas, small_rect, pos.x, pos.y, pos.x + fillPercentage * size.x,
      pos.y + size.y );
   Tcl_GlobalEval( interp, buf );
};

//==============================================================================

Gauge::Gauge( Vertex* v, MN_Point* p, MN_Point* s, char* l )
: Mark( v, p, s, l )
{
   sprintf( buf, "%s create oval %g %g %g %g -fill #%02x%02x%02x -tag node",
      canvas, pos.x, pos.y, pos.x + size.x, pos.y + size.y,
      (int)(255.0 * blank.x), (int)(255.0 * blank.y), (int)(255.0 * blank.z) );
   Tcl_GlobalEval( interp, buf );
   circleA = atoi( interp->result );

   sprintf( buf, "%s create arc %g %g %g %g -fill #%02x%02x%02x -tag node",
      canvas, pos.x, pos.y, pos.x + size.x, pos.y + size.y,
      (int)(255.0 * fill.x), (int)(255.0 * fill.y), (int)(255.0 * fill.z) );
   Tcl_GlobalEval( interp, buf );
   fillA = atoi( interp->result );

   sprintf( buf, "%s create text %g %g -text {%s} -anchor nw",
      canvas, pos.x + size.x + 5, pos.y, label );
   Tcl_GlobalEval( interp, buf );
   label_widget = atoi( interp->result );

   sprintf( buf, "set idFromW(%d) %x", circleA, (unsigned int)v );
   Tcl_GlobalEval( interp, buf );
};

virtual void Gauge::SetPos( MN_Point *p )
{
   pos = *p;
   Display();
};

virtual void Gauge::SetSize( MN_Point *p )
{
   size = *p;
   Display();
};

virtual void Gauge::SetFill( MN_Color *p )
{
   fill = *p;
   sprintf( buf, "%s itemconfigure %d -fill #%02x%02x%02x",
      canvas, fillA, fill.x, fill.y, fill.z );
   Tcl_GlobalEval( interp, buf );
};

virtual void Gauge::SetBlank( MN_Color *p )
{
   blank = *p;
   sprintf( buf, "%s itemconfigure %d -fill #%02x%02x%02x",
      canvas, circleA, blank.x, blank.y, blank.z );
   Tcl_GlobalEval( interp, buf );
};

virtual void Gauge::SetOutline( MN_Color *p )
{
   outline = *p;
   sprintf( buf, "%s itemconfigure %d -outline #%02x%02x%02x",
      canvas, circleA, outline.x, outline.y, outline.z );
   Tcl_GlobalEval( interp, buf );
};

virtual void Gauge::SetThickness( float p )
{
   thickness = p;
   sprintf( buf, "%s itemconfigure %d -width %g",
      canvas, circleA, 1.0 + 5.0 * p );
   Tcl_GlobalEval( interp, buf );
};

virtual void Gauge::Display( void )
{
   sprintf( buf, "%s coords %d %g %g %g %g",
      canvas, fillA, pos.x, pos.y, pos.x + size.x, pos.y + size.y );
   Tcl_GlobalEval( interp, buf );

   sprintf( buf, "%s itemconfigure %d -extent %d",
      canvas, fillA, (int)(fillPercentage * 360.0) );
   Tcl_GlobalEval( interp, buf );

   sprintf( buf, "%s coords %d %g %g %g %g",
      canvas, circleA, pos.x, pos.y, pos.x + size.x, pos.y + size.y );
   Tcl_GlobalEval( interp, buf );

   sprintf( buf, "%s coords %d %g %g",
      canvas, label_widget, pos.x + size.x + 5, pos.y );
   Tcl_GlobalEval( interp, buf );
};

virtual void Gauge::Show( bool show )
{
   if( show )
   {
      if( pos.x < 0 )
      {
         ShowWidget( canvas, interp, fillA,        true );
         ShowWidget( canvas, interp, circleA,      true );
         ShowWidget( canvas, interp, label_widget, true );
      }
   }
   else   // hide
   {
      if( pos.x >= 0 )
      {
         ShowWidget( canvas, interp, fillA,        false );
         ShowWidget( canvas, interp, circleA,      false );
         ShowWidget( canvas, interp, label_widget, false );
      }
   }
};

virtual void Gauge::SetFillPercentage( float p )
{
   fillPercentage = p;
   sprintf( buf, "%s itemconfigure %d -extent %d",
      canvas, fillA, (int)(fillPercentage * 360.0) );
   Tcl_GlobalEval( interp, buf );
};

//==============================================================================

Icon::Icon( Vertex* v, MN_Point* p, MN_Point* s, char* l )
: Mark( v, p, s, l )
{
   sprintf( buf,
      "%s create rectangle %g %g %g %g -fill #%02x%02x%02x -tag node",
      canvas, pos.x, pos.y, pos.x + size.x, pos.y + size.y,
      (int)(255.0 * fill.x), (int)(255.0 * fill.y), (int)(255.0 * fill.z) );
   Tcl_GlobalEval( interp, buf );
   rect = atoi( interp->result );

   sprintf( buf, "%s create bitmap %g %g -anchor nw -tag node",
      canvas, pos.x, pos.y );
   Tcl_GlobalEval( interp, buf );
   bitmap = atoi( interp->result );

   sprintf( buf, "%s create text %g %g -text {%s} -anchor nw",
      canvas, pos.x + size.x + 5, pos.y, label );
   Tcl_GlobalEval( interp, buf );
   label_widget = atoi( interp->result );

   sprintf( buf, "set idFromW(%d) %x", rect, (unsigned int)v );
   Tcl_GlobalEval( interp, buf );
};

void Icon::SetBitmap( char* id )
{
   sprintf( buf,
      "%s configure %d -background white -bitmap @bitmaps(%s,$state(size))",
      canvas, bitmap, id );
   Tcl_GlobalEval( interp, buf );
};

virtual void Icon::SetPos( MN_Point *p )
{
   pos = *p;
   Display();
};

virtual void Icon::SetSize( MN_Point *p )
{
   size = *p;
   Display();
};

virtual void Icon::SetFill( MN_Color *p )
{
   fill = *p;
   sprintf( buf, "%s itemconfigure %d -fill #%02x%02x%02x",
      canvas, rect, fill.x, fill.y, fill.z );
   Tcl_GlobalEval( interp, buf );
};

virtual void Icon::SetOutline( MN_Color *p )
{
   outline = *p;
   sprintf( buf, "%s itemconfigure %d -outline #%02x%02x%02x",
      canvas, rect, outline.x, outline.y, outline.z );
   Tcl_GlobalEval( interp, buf );
};

virtual void Icon::SetThickness( float p )
{
   thickness = p;
   sprintf( buf, "%s itemconfigure %d -width %g",
      canvas, rect, 1.0 + 5.0 * p );
   Tcl_GlobalEval( interp, buf );
};

virtual void Icon::Display( void )
{
   sprintf( buf, "%s coords %d %g %g %g %g",
      canvas, rect, pos.x, pos.y, pos.x + size.x, pos.y + size.y );
   Tcl_GlobalEval( interp, buf );

   sprintf( buf, "%s coords %d %g %g",
      canvas, bitmap, pos.x, pos.y );
   Tcl_GlobalEval( interp, buf );

   sprintf( buf, "%s coords %d %g %g",
      canvas, label_widget, pos.x + size.x + 5, pos.y );
   Tcl_GlobalEval( interp, buf );
};

virtual void Icon::Show( bool show )
{
   if( show )
   {
      if( pos.x < 0 )
      {
         ShowWidget( canvas, interp, bitmap, true );
         ShowWidget( canvas, interp, rect,   true );
      }
   }
   else   // hide
   {
      if( pos.x >= 0 )
      {
         ShowWidget( canvas, interp, bitmap, false );
         ShowWidget( canvas, interp, rect,   false );
      }
   }
};

//==============================================================================
